package sun.font;

import java.security.AccessController;
import java.security.PrivilegedAction;
import sun.misc.InnocuousThread;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;
import java.awt.geom.Point2D;
import java.lang.ref.WeakReference;

class T2KFontScaler extends FontScaler
{
    private int[] bwGlyphs;
    private static final int TRUETYPE_FONT = 1;
    private static final int TYPE1_FONT = 2;
    private long layoutTablePtr;
    
    private void initBWGlyphs() {
        if (this.font.get() != null && "Courier New".equals(((Font2D)this.font.get()).getFontName(null))) {
            this.bwGlyphs = new int[2];
            CharToGlyphMapper mapper = ((Font2D)this.font.get()).getMapper();
            this.bwGlyphs[0] = mapper.charToGlyph('W');
            this.bwGlyphs[1] = mapper.charToGlyph('w');
        }
    }
    
    private static native void initIDs(Class p0);
    
    private void invalidateScaler() throws FontScalerException {
        this.nativeScaler = 0L;
        this.font = null;
        throw new FontScalerException();
    }
    
    public T2KFontScaler(Font2D referent, int n, boolean b, int n2) {
        this.layoutTablePtr = 0L;
        int n3 = 1;
        if ((referent instanceof Type1Font)) {
            n3 = 2;
        }
        this.font = new WeakReference(referent);
        this.initBWGlyphs();
        this.nativeScaler = this.initNativeScaler(referent, n3, n, b, n2, this.bwGlyphs);
    }
    
    @Override
    synchronized StrikeMetrics getFontMetrics(long n) throws FontScalerException {
        if (this.nativeScaler != 0L) {
            return this.getFontMetricsNative((Font2D)this.font.get(), n, this.nativeScaler);
        }
        return FontScaler.getNullScaler().getFontMetrics(0L);
    }
    
    @Override
    synchronized float getGlyphAdvance(long n, int n2) throws FontScalerException {
        if (this.nativeScaler != 0L) {
            return this.getGlyphAdvanceNative((Font2D)this.font.get(), n, this.nativeScaler, n2);
        }
        return FontScaler.getNullScaler().getGlyphAdvance(0L, n2);
    }
    
    @Override
    synchronized void getGlyphMetrics(long n, int n2, Point2D.Float float1) throws FontScalerException {
        if (this.nativeScaler != 0L) {
            this.getGlyphMetricsNative((Font2D)this.font.get(), n, this.nativeScaler, n2, float1);
        }
        else {
            FontScaler.getNullScaler().getGlyphMetrics(0L, n2, float1);
        }
    }
    
    @Override
    synchronized long getGlyphImage(long n, int n2) throws FontScalerException {
        if (this.nativeScaler != 0L) {
            return this.getGlyphImageNative((Font2D)this.font.get(), n, this.nativeScaler, n2);
        }
        return FontScaler.getNullScaler().getGlyphImage(0L, n2);
    }
    
    @Override
    synchronized Rectangle2D.Float getGlyphOutlineBounds(long n, int n2) throws FontScalerException {
        if (this.nativeScaler != 0L) {
            return this.getGlyphOutlineBoundsNative((Font2D)this.font.get(), n, this.nativeScaler, n2);
        }
        return FontScaler.getNullScaler().getGlyphOutlineBounds(0L, n2);
    }
    
    @Override
    synchronized GeneralPath getGlyphOutline(long n, int n2, float n3, float n4) throws FontScalerException {
        if (this.nativeScaler != 0L) {
            return this.getGlyphOutlineNative((Font2D)this.font.get(), n, this.nativeScaler, n2, n3, n4);
        }
        return FontScaler.getNullScaler().getGlyphOutline(0L, n2, n3, n4);
    }
    
    @Override
    synchronized GeneralPath getGlyphVectorOutline(long n, int[] array, int n2, float n3, float n4) throws FontScalerException {
        if (this.nativeScaler != 0L) {
            return this.getGlyphVectorOutlineNative((Font2D)this.font.get(), n, this.nativeScaler, array, n2, n3, n4);
        }
        return FontScaler.getNullScaler().getGlyphVectorOutline(0L, array, n2, n3, n4);
    }
    
    @Override
    synchronized int getNumGlyphs() throws FontScalerException {
        if (this.nativeScaler != 0L) {
            return this.getNumGlyphsNative(this.nativeScaler);
        }
        return FontScaler.getNullScaler().getNumGlyphs();
    }
    
    @Override
    synchronized int getMissingGlyphCode() throws FontScalerException {
        if (this.nativeScaler != 0L) {
            return this.getMissingGlyphCodeNative(this.nativeScaler);
        }
        return FontScaler.getNullScaler().getMissingGlyphCode();
    }
    
    @Override
    synchronized int getGlyphCode(char c) throws FontScalerException {
        if (this.nativeScaler != 0L) {
            return this.getGlyphCodeNative(this.nativeScaler, c);
        }
        return FontScaler.getNullScaler().getGlyphCode(c);
    }
    
    @Override
    synchronized long getLayoutTableCache() throws FontScalerException {
        if (this.nativeScaler == 0L) {
            return FontScaler.getNullScaler().getLayoutTableCache();
        }
        if (this.layoutTablePtr == 0L) {
            this.layoutTablePtr = this.getLayoutTableCacheNative(this.nativeScaler);
        }
        return this.layoutTablePtr;
    }
    
    private synchronized void disposeScaler() {
        this.disposeNativeScaler(this.nativeScaler, this.layoutTablePtr);
        this.nativeScaler = 0L;
        this.layoutTablePtr = 0L;
    }
    
    @Override
    public synchronized void dispose() {
        if (this.nativeScaler != 0L || this.layoutTablePtr != 0L) {
            new InnocuousThread(new Runnable() {
                @Override
                public void run() {
                    T2KFontScaler.access$000(T2KFontScaler.this);
                }
            }).start();
        }
    }
    
    @Override
    synchronized Point2D.Float getGlyphPoint(long n, int n2, int n3) throws FontScalerException {
        if (this.nativeScaler != 0L) {
            return this.getGlyphPointNative((Font2D)this.font.get(), n, this.nativeScaler, n2, n3);
        }
        return FontScaler.getNullScaler().getGlyphPoint(n, n2, n3);
    }
    
    @Override
    synchronized long getUnitsPerEm() {
        if (this.nativeScaler != 0L) {
            return this.getUnitsPerEMNative(this.nativeScaler);
        }
        return FontScaler.getNullScaler().getUnitsPerEm();
    }
    
    @Override
    synchronized long createScalerContext(double[] array, int n, int n2, float n3, float n4, boolean b) {
        if (this.nativeScaler != 0L) {
            return this.createScalerContextNative(this.nativeScaler, array, n, n2, n3, n4, b);
        }
        return NullFontScaler.getNullScalerContext();
    }
    
    private native long initNativeScaler(Font2D p0, int p1, int p2, boolean p3, int p4, int[] p5);
    
    private native StrikeMetrics getFontMetricsNative(Font2D p0, long p1, long p2);
    
    private native float getGlyphAdvanceNative(Font2D p0, long p1, long p2, int p3);
    
    private native void getGlyphMetricsNative(Font2D p0, long p1, long p2, int p3, Point2D.Float p4);
    
    private native long getGlyphImageNative(Font2D p0, long p1, long p2, int p3);
    
    private native Rectangle2D.Float getGlyphOutlineBoundsNative(Font2D p0, long p1, long p2, int p3);
    
    private native GeneralPath getGlyphOutlineNative(Font2D p0, long p1, long p2, int p3, float p4, float p5);
    
    private native GeneralPath getGlyphVectorOutlineNative(Font2D p0, long p1, long p2, int[] p3, int p4, float p5, float p6);
    
    private native int getGlyphCodeNative(long p0, char p1);
    
    private native long getLayoutTableCacheNative(long p0);
    
    private native void disposeNativeScaler(long p0, long p1);
    
    private native int getNumGlyphsNative(long p0);
    
    private native int getMissingGlyphCodeNative(long p0);
    
    private native long getUnitsPerEMNative(long p0);
    
    private native long createScalerContextNative(long p0, double[] p1, int p2, int p3, float p4, float p5, boolean p6);
    
    private native Point2D.Float getGlyphPointNative(Font2D p0, long p1, long p2, int p3, int p4);
    
    @Override
    void invalidateScalerContext(long n) {
    }
    
    static void access$000(T2KFontScaler t2KFontScaler) {
        t2KFontScaler.disposeScaler();
    }
    
    static {
        AccessController.doPrivileged(new PrivilegedAction() {
            @Override
            public Object run() {
                FontManagerNativeLibrary.load();
                System.loadLibrary("t2k");
                return null;
            }
        });
        initIDs(T2KFontScaler.class);
    }
}
